searchData={"items":[{"type":"module","title":"FalEx","doc":"An Elixir client for fal.ai.\n\nFalEx provides a simple interface to run AI models on fal.ai's\nserverless platform. It supports synchronous execution, queued jobs,\nstreaming responses, and real-time communication.","ref":"FalEx.html"},{"type":"module","title":"Installation - FalEx","doc":"Add `fal_ex` to your list of dependencies in `mix.exs`:\n\n    def deps do\n      [\n        {:fal_ex, \"~> 0.1.0\"}\n      ]\n    end","ref":"FalEx.html#module-installation"},{"type":"module","title":"Configuration - FalEx","doc":"The recommended way to configure FalEx is through your application configuration:\n\n    # config/config.exs\n    config :fal_ex,\n      api_key: \"your-api-key\"\n\n    # config/runtime.exs (for runtime configuration)\n    import Config\n\n    if config_env() == :prod do\n      config :fal_ex,\n        api_key: System.fetch_env!(\"FAL_KEY\")\n    end\n\nYou can also use environment variables (these override application config):\n\n    export FAL_KEY=\"your-api-key\"\n\nOr configure programmatically at runtime:\n\n    FalEx.config(credentials: \"your-api-key\")","ref":"FalEx.html#module-configuration"},{"type":"module","title":"Basic Usage - FalEx","doc":"# Simple synchronous execution\n    {:ok, result} = FalEx.run(\"fal-ai/fast-sdxl\",\n      input: %{\n        prompt: \"A cute cat wearing a hat\"\n      }\n    )\n\n    # Queue-based execution with status updates\n    {:ok, result} = FalEx.subscribe(\"fal-ai/fast-sdxl\",\n      input: %{prompt: \"A mountain landscape\"},\n      on_queue_update: fn status -> IO.inspect(status) end\n    )\n\n    # Streaming responses\n    {:ok, stream} = FalEx.stream(\"fal-ai/flux/dev\",\n      input: %{prompt: \"A cute robot\", image_size: \"square\"}\n    )\n\n    stream\n    |> Stream.each(fn chunk -> IO.inspect(chunk) end)\n    |> Stream.run()","ref":"FalEx.html#module-basic-usage"},{"type":"function","title":"FalEx.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"FalEx.html#child_spec/1"},{"type":"function","title":"FalEx.config/1","doc":"Configures the global FalEx client at runtime.\n\nNote: The recommended way to configure FalEx is through application configuration.\nUse this function only when you need to override configuration at runtime.","ref":"FalEx.html#config/1"},{"type":"function","title":"Options - FalEx.config/1","doc":"See `FalEx.Config.new/1` for available options.","ref":"FalEx.html#config/1-options"},{"type":"function","title":"Examples - FalEx.config/1","doc":"# Override configuration at runtime\n    FalEx.config(credentials: System.get_env(\"FAL_KEY\"))\n\n    # Configure with multiple options\n    FalEx.config(\n      credentials: {\"key_id\", \"key_secret\"},\n      proxy_url: \"/api/fal/proxy\"\n    )","ref":"FalEx.html#config/1-examples"},{"type":"function","title":"Application Configuration (Recommended) - FalEx.config/1","doc":"# In config/config.exs or config/runtime.exs\n    config :fal_ex,\n      api_key: System.get_env(\"FAL_KEY\"),\n      base_url: \"https://fal.run\"","ref":"FalEx.html#config/1-application-configuration-recommended"},{"type":"function","title":"FalEx.queue/0","doc":"Access the queue client for advanced queue operations.","ref":"FalEx.html#queue/0"},{"type":"function","title":"FalEx.realtime/0","doc":"Access the realtime client for WebSocket connections.","ref":"FalEx.html#realtime/0"},{"type":"function","title":"FalEx.run/2","doc":"Runs a fal endpoint synchronously.","ref":"FalEx.html#run/2"},{"type":"function","title":"Parameters - FalEx.run/2","doc":"* `endpoint_id` - The endpoint identifier (e.g., \"fal-ai/fast-sdxl\")\n  * `opts` - Run options including `:input`","ref":"FalEx.html#run/2-parameters"},{"type":"function","title":"Options - FalEx.run/2","doc":"* `:input` - The input payload for the model\n  * `:path` - Optional path to append to the endpoint\n  * `:method` - HTTP method (default: `:post`)\n  * `:abort_signal` - Reference to abort the request","ref":"FalEx.html#run/2-options"},{"type":"function","title":"Examples - FalEx.run/2","doc":"{:ok, result} = FalEx.run(\"fal-ai/fast-sdxl\",\n      input: %{\n        prompt: \"A beautiful sunset\",\n        image_size: \"square\"\n      }\n    )","ref":"FalEx.html#run/2-examples"},{"type":"function","title":"Notes - FalEx.run/2","doc":"This function blocks until the model execution completes. For long-running\nmodels, consider using `subscribe/2` instead.","ref":"FalEx.html#run/2-notes"},{"type":"type","title":"FalEx.run_result/0","doc":"","ref":"FalEx.html#t:run_result/0"},{"type":"function","title":"FalEx.start_link/1","doc":"Starts the FalEx client process.\n\nThis is typically called automatically by the application supervisor.","ref":"FalEx.html#start_link/1"},{"type":"function","title":"FalEx.storage/0","doc":"Access the storage client for file operations.","ref":"FalEx.html#storage/0"},{"type":"function","title":"FalEx.stream/2","doc":"Streams partial results from a streaming-capable endpoint.\n\nReturns `{:ok, stream}` where `stream` is an Elixir Stream that yields data chunks as they arrive.","ref":"FalEx.html#stream/2"},{"type":"function","title":"Parameters - FalEx.stream/2","doc":"* `endpoint_id` - The endpoint identifier\n  * `opts` - Stream options including `:input`","ref":"FalEx.html#stream/2-parameters"},{"type":"function","title":"Examples - FalEx.stream/2","doc":"# Stream image generation with progress updates\n    {:ok, stream} = FalEx.stream(\"fal-ai/flux/dev\",\n      input: %{\n        prompt: \"a cat\",\n        seed: 6252023,\n        image_size: \"landscape_4_3\",\n        num_images: 4\n      }\n    )\n\n    # Process each event as it arrives\n    stream\n    |> Stream.each(fn event ->\n      IO.inspect(event, label: \"Stream event\")\n    end)\n    |> Stream.run()\n\n    # Or collect all events and get the final result\n    case FalEx.stream(\"fal-ai/flux/dev\", input: %{prompt: \"a cute robot\"}) do\n      {:ok, stream} ->\n        events = Enum.to_list(stream)\n        IO.puts(\"Received #{length(events)} events\")\n        # The last event typically contains the final result\n\n      {:error, reason} ->\n        IO.puts(\"Error: #{inspect(reason)}\")\n    end","ref":"FalEx.html#stream/2-examples"},{"type":"function","title":"FalEx.streaming/0","doc":"Access the streaming client for advanced streaming operations.","ref":"FalEx.html#streaming/0"},{"type":"function","title":"FalEx.subscribe/2","doc":"Submits a request to the queue and subscribes to status updates.","ref":"FalEx.html#subscribe/2"},{"type":"function","title":"Parameters - FalEx.subscribe/2","doc":"* `endpoint_id` - The endpoint identifier\n  * `opts` - Run options plus queue-specific options","ref":"FalEx.html#subscribe/2-parameters"},{"type":"function","title":"Options - FalEx.subscribe/2","doc":"All options from `run/2` plus:\n\n  * `:on_queue_update` - Callback for status updates\n  * `:logs` - Include logs in status updates (default: false)\n  * `:webhook_url` - URL for webhook notifications\n  * `:poll_interval` - Status poll interval in ms (default: 500)","ref":"FalEx.html#subscribe/2-options"},{"type":"function","title":"Examples - FalEx.subscribe/2","doc":"{:ok, result} = FalEx.subscribe(\"fal-ai/fast-sdxl\",\n      input: %{prompt: \"A cute dog\"},\n      logs: true,\n      on_queue_update: fn status ->\n        IO.puts(\"Status: \" <> inspect(status.status))\n      end\n    )","ref":"FalEx.html#subscribe/2-examples"},{"type":"module","title":"FalEx.Auth","doc":"Authentication module for FalEx client.\n\nHandles API key authentication and temporary token generation\nfor browser/realtime clients.","ref":"FalEx.Auth.html"},{"type":"function","title":"FalEx.Auth.create_temporary_token/2","doc":"Generates a temporary authentication token for browser/realtime usage.\n\nThis is used when connecting to streaming or WebSocket endpoints from\nenvironments where exposing the main API key would be insecure.","ref":"FalEx.Auth.html#create_temporary_token/2"},{"type":"function","title":"Options - FalEx.Auth.create_temporary_token/2","doc":"* `:allowed_apps` - List of allowed app IDs (default: [\"*\"])\n  * `:token_expiration` - Token expiration time in seconds (default: 120)","ref":"FalEx.Auth.html#create_temporary_token/2-options"},{"type":"function","title":"Examples - FalEx.Auth.create_temporary_token/2","doc":"iex> {:ok, token} = FalEx.Auth.create_temporary_token(config)\n    iex> is_binary(token)\n    true","ref":"FalEx.Auth.html#create_temporary_token/2-examples"},{"type":"function","title":"FalEx.Auth.get_request_headers/1","doc":"Gets the base authentication headers for a request.\n\nIncludes credentials if present and not using a proxy.","ref":"FalEx.Auth.html#get_request_headers/1"},{"type":"function","title":"FalEx.Auth.has_credentials?/1","doc":"Checks if credentials are present in the configuration.","ref":"FalEx.Auth.html#has_credentials?/1"},{"type":"function","title":"FalEx.Auth.to_headers/1","doc":"Generates authentication headers based on the provided credentials.","ref":"FalEx.Auth.html#to_headers/1"},{"type":"function","title":"Examples - FalEx.Auth.to_headers/1","doc":"iex> FalEx.Auth.to_headers(\"key_123456\")\n    [{\"Authorization\", \"Key key_123456\"}]\n    \n    iex> FalEx.Auth.to_headers({\"id_123\", \"secret_456\"})\n    [{\"Authorization\", \"Key id_123:secret_456\"}]","ref":"FalEx.Auth.html#to_headers/1-examples"},{"type":"module","title":"FalEx.Client","doc":"The main client implementation for FalEx.\n\nThis module creates and manages the various sub-clients (queue, storage, streaming, realtime)\nand provides the core run/subscribe functionality.","ref":"FalEx.Client.html"},{"type":"function","title":"FalEx.Client.create/1","doc":"Creates a new FalEx client with the given configuration.","ref":"FalEx.Client.html#create/1"},{"type":"function","title":"FalEx.Client.run/3","doc":"Runs a fal endpoint synchronously.","ref":"FalEx.Client.html#run/3"},{"type":"function","title":"FalEx.Client.subscribe/3","doc":"Submits a request to the queue and subscribes to updates.","ref":"FalEx.Client.html#subscribe/3"},{"type":"type","title":"FalEx.Client.t/0","doc":"","ref":"FalEx.Client.html#t:t/0"},{"type":"module","title":"FalEx.Config","doc":"Configuration module for FalEx client.\n\nHandles authentication credentials and client configuration.\nSupports environment variables and runtime configuration.","ref":"FalEx.Config.html"},{"type":"type","title":"FalEx.Config.credentials/0","doc":"","ref":"FalEx.Config.html#t:credentials/0"},{"type":"function","title":"FalEx.Config.get_base_url/1","doc":"Gets the effective base URL, considering proxy configuration.","ref":"FalEx.Config.html#get_base_url/1"},{"type":"function","title":"FalEx.Config.new/1","doc":"Creates a new configuration.","ref":"FalEx.Config.html#new/1"},{"type":"function","title":"Options - FalEx.Config.new/1","doc":"* `:credentials` - API credentials. Can be:\n    - A string API key\n    - A tuple of `{key_id, key_secret}`\n    - A function that returns either of the above\n  * `:proxy_url` - URL of a proxy server to route requests through\n  * `:request_middleware` - Custom middleware module for request modification\n  * `:response_handler` - Custom response handler function\n  * `:fetch_implementation` - Custom HTTP client implementation\n  * `:base_url` - Base URL for API requests (default: \"https://fal.run\")","ref":"FalEx.Config.html#new/1-options"},{"type":"function","title":"Examples - FalEx.Config.new/1","doc":"# Using API key from environment\n    FalEx.Config.new(credentials: System.get_env(\"FAL_KEY\"))\n\n    # Using key ID and secret\n    FalEx.Config.new(credentials: {\"key_id\", \"key_secret\"})\n\n    # Using a resolver function\n    FalEx.Config.new(credentials: fn -> System.get_env(\"FAL_KEY\") end)\n\n    # Using proxy\n    FalEx.Config.new(proxy_url: \"/api/fal/proxy\")","ref":"FalEx.Config.html#new/1-examples"},{"type":"function","title":"FalEx.Config.resolve_credentials/1","doc":"Resolves credentials at runtime.\n\nHandles credential resolver functions and returns the actual credentials.","ref":"FalEx.Config.html#resolve_credentials/1"},{"type":"type","title":"FalEx.Config.t/0","doc":"","ref":"FalEx.Config.html#t:t/0"},{"type":"function","title":"FalEx.Config.using_proxy?/1","doc":"Checks if the configuration uses a proxy.","ref":"FalEx.Config.html#using_proxy?/1"},{"type":"module","title":"FalEx.Queue","doc":"Queue client for managing asynchronous fal.ai requests.\n\nProvides functionality to submit jobs to the queue, monitor their status,\nand retrieve results when complete.","ref":"FalEx.Queue.html"},{"type":"function","title":"FalEx.Queue.cancel/3","doc":"Cancels a queued request.","ref":"FalEx.Queue.html#cancel/3"},{"type":"function","title":"FalEx.Queue.create/1","doc":"Creates a new queue client.","ref":"FalEx.Queue.html#create/1"},{"type":"function","title":"FalEx.Queue.result/3","doc":"Gets the result of a completed request.","ref":"FalEx.Queue.html#result/3"},{"type":"function","title":"FalEx.Queue.status/3","doc":"Gets the status of a queued request.","ref":"FalEx.Queue.html#status/3"},{"type":"function","title":"Options - FalEx.Queue.status/3","doc":"* `:request_id` - The request ID to check\n  * `:logs` - Include execution logs (default: false)","ref":"FalEx.Queue.html#status/3-options"},{"type":"function","title":"FalEx.Queue.submit/3","doc":"Submits a request to the queue.","ref":"FalEx.Queue.html#submit/3"},{"type":"function","title":"Options - FalEx.Queue.submit/3","doc":"* `:input` - The input payload for the model\n  * `:webhook_url` - Optional webhook URL for completion notification","ref":"FalEx.Queue.html#submit/3-options"},{"type":"function","title":"Examples - FalEx.Queue.submit/3","doc":"{:ok, %{\"request_id\" => request_id}} = Queue.submit(queue, \"fal-ai/fast-sdxl\", \n      input: %{prompt: \"A cat\"}\n    )","ref":"FalEx.Queue.html#submit/3-examples"},{"type":"function","title":"FalEx.Queue.subscribe_to_status/3","doc":"Subscribes to status updates for a queued request.\n\nPolls or uses SSE to monitor the request status until completion.","ref":"FalEx.Queue.html#subscribe_to_status/3"},{"type":"function","title":"Options - FalEx.Queue.subscribe_to_status/3","doc":"* `:request_id` - The request ID to monitor\n  * `:logs` - Include logs in status updates\n  * `:on_queue_update` - Callback function for status updates\n  * `:poll_interval` - Polling interval in ms (default: 500)\n  * `:timeout` - Maximum time to wait in ms (default: 300000)","ref":"FalEx.Queue.html#subscribe_to_status/3-options"},{"type":"type","title":"FalEx.Queue.t/0","doc":"","ref":"FalEx.Queue.html#t:t/0"},{"type":"module","title":"FalEx.Realtime","doc":"Realtime client for WebSocket connections to fal.ai.\n\nProvides bidirectional communication with AI models that support\nreal-time interaction.","ref":"FalEx.Realtime.html"},{"type":"function","title":"FalEx.Realtime.close/1","doc":"Closes a realtime connection.","ref":"FalEx.Realtime.html#close/1"},{"type":"function","title":"FalEx.Realtime.connect/3","doc":"Connects to a realtime endpoint.","ref":"FalEx.Realtime.html#connect/3"},{"type":"function","title":"Options - FalEx.Realtime.connect/3","doc":"* `:connection_key` - Reuse existing connection with this key\n  * `:throttle_interval` - Throttle messages (ms, default: 128)\n  * `:on_message` - Callback for incoming messages\n  * `:on_error` - Callback for errors\n  * `:on_close` - Callback for connection close","ref":"FalEx.Realtime.html#connect/3-options"},{"type":"function","title":"Examples - FalEx.Realtime.connect/3","doc":"{:ok, conn} = Realtime.connect(realtime, \"fal-ai/llava-next\",\n      on_message: fn message ->\n        IO.inspect(message)\n      end\n    )","ref":"FalEx.Realtime.html#connect/3-examples"},{"type":"function","title":"FalEx.Realtime.create/1","doc":"Creates a new realtime client.","ref":"FalEx.Realtime.html#create/1"},{"type":"function","title":"FalEx.Realtime.send/2","doc":"Sends a message through the realtime connection.","ref":"FalEx.Realtime.html#send/2"},{"type":"type","title":"FalEx.Realtime.t/0","doc":"","ref":"FalEx.Realtime.html#t:t/0"},{"type":"module","title":"FalEx.Request","doc":"HTTP request handling for FalEx client.\n\nBuilds URLs, dispatches requests, and handles responses using Tesla.","ref":"FalEx.Request.html"},{"type":"function","title":"FalEx.Request.build_url/2","doc":"Builds a URL for the given endpoint ID and options.","ref":"FalEx.Request.html#build_url/2"},{"type":"function","title":"Examples - FalEx.Request.build_url/2","doc":"iex> FalEx.Request.build_url(\"fal-ai/fast-sdxl\", %{})\n    \"https://fal.run/fal-ai/fast-sdxl\"\n\n    iex> FalEx.Request.build_url(\"12345\", %{subdomain: \"queue\"})\n    \"https://queue.fal.run/12345\"","ref":"FalEx.Request.html#build_url/2-examples"},{"type":"function","title":"FalEx.Request.dispatch_request/1","doc":"Dispatches an HTTP request.","ref":"FalEx.Request.html#dispatch_request/1"},{"type":"function","title":"Options - FalEx.Request.dispatch_request/1","doc":"* `:method` - HTTP method (default: :post)\n  * `:input` - Request payload\n  * `:config` - FalEx configuration\n  * `:options` - Additional request options (headers, timeout, etc.)","ref":"FalEx.Request.html#dispatch_request/1-options"},{"type":"function","title":"FalEx.Request.parse_endpoint_id/1","doc":"Parses an endpoint ID into its components.","ref":"FalEx.Request.html#parse_endpoint_id/1"},{"type":"function","title":"Examples - FalEx.Request.parse_endpoint_id/1","doc":"iex> FalEx.Request.parse_endpoint_id(\"fal-ai/fast-sdxl\")\n    %{owner: \"fal-ai\", alias: \"fast-sdxl\"}\n\n    iex> FalEx.Request.parse_endpoint_id(\"12345\")\n    %{path: \"12345\"}","ref":"FalEx.Request.html#parse_endpoint_id/1-examples"},{"type":"module","title":"FalEx.Response","doc":"Response handling and error types for FalEx client.","ref":"FalEx.Response.html"},{"type":"function","title":"FalEx.Response.api_error/3","doc":"Creates an API error from response data.","ref":"FalEx.Response.html#api_error/3"},{"type":"function","title":"FalEx.Response.completed_queue_status?/1","doc":"Checks if a queue status indicates completion.","ref":"FalEx.Response.html#completed_queue_status?/1"},{"type":"function","title":"FalEx.Response.queue_status?/1","doc":"Checks if a response is a queue status response.","ref":"FalEx.Response.html#queue_status?/1"},{"type":"function","title":"FalEx.Response.result_response_handler/1","doc":"Result response handler that wraps the response data.\n\nThis is the default response handler that formats successful\nresponses into the standard result format.","ref":"FalEx.Response.html#result_response_handler/1"},{"type":"function","title":"FalEx.Response.validation_error/1","doc":"Creates a validation error from response data.","ref":"FalEx.Response.html#validation_error/1"},{"type":"module","title":"FalEx.Storage","doc":"Storage client for handling file uploads and transformations.\n\nAutomatically uploads files to fal.ai storage and transforms input\npayloads to replace file references with URLs.","ref":"FalEx.Storage.html"},{"type":"function","title":"FalEx.Storage.create/1","doc":"Creates a new storage client.","ref":"FalEx.Storage.html#create/1"},{"type":"type","title":"FalEx.Storage.t/0","doc":"","ref":"FalEx.Storage.html#t:t/0"},{"type":"function","title":"FalEx.Storage.transform_input/2","doc":"Transforms input data by uploading any file references.\n\nRecursively processes the input, uploading any binary data or file paths\nand replacing them with URLs.","ref":"FalEx.Storage.html#transform_input/2"},{"type":"function","title":"Examples - FalEx.Storage.transform_input/2","doc":"input = %{\n      prompt: \"A cat\",\n      image: File.read!(\"cat.jpg\")\n    }\n    \n    transformed = Storage.transform_input(storage, input)\n    # => %{prompt: \"A cat\", image: \"https://fal.media/...\"}","ref":"FalEx.Storage.html#transform_input/2-examples"},{"type":"function","title":"FalEx.Storage.upload/3","doc":"Uploads a file to fal.ai storage.","ref":"FalEx.Storage.html#upload/3"},{"type":"function","title":"Parameters - FalEx.Storage.upload/3","doc":"* `data` - Binary file data or a file path\n  * `opts` - Upload options","ref":"FalEx.Storage.html#upload/3-parameters"},{"type":"function","title":"Options - FalEx.Storage.upload/3","doc":"* `:file_name` - Name of the file\n  * `:content_type` - MIME type of the file","ref":"FalEx.Storage.html#upload/3-options"},{"type":"function","title":"Examples - FalEx.Storage.upload/3","doc":"{:ok, url} = Storage.upload(storage, File.read!(\"image.png\"),\n      file_name: \"image.png\",\n      content_type: \"image/png\"\n    )","ref":"FalEx.Storage.html#upload/3-examples"},{"type":"module","title":"FalEx.Streaming","doc":"Streaming client for handling Server-Sent Events (SSE) from fal.ai.\n\nProvides real-time streaming of model outputs with support for partial results.","ref":"FalEx.Streaming.html"},{"type":"function","title":"FalEx.Streaming.close/1","doc":"Closes a stream.","ref":"FalEx.Streaming.html#close/1"},{"type":"function","title":"FalEx.Streaming.create/1","doc":"Creates a new streaming client.","ref":"FalEx.Streaming.html#create/1"},{"type":"function","title":"FalEx.Streaming.next/1","doc":"Gets the next chunk from a stream.\n\nReturns:\n  * `{:ok, data, stream}` - Data chunk received\n  * `{:done, stream}` - Stream completed\n  * `{:error, reason}` - Error occurred","ref":"FalEx.Streaming.html#next/1"},{"type":"function","title":"FalEx.Streaming.stream/3","doc":"Starts a streaming request to a fal endpoint.\n\nReturns a stream handle that can be used to receive data chunks.","ref":"FalEx.Streaming.html#stream/3"},{"type":"function","title":"Options - FalEx.Streaming.stream/3","doc":"* `:input` - Input payload for the model\n  * `:method` - HTTP method (default: :post)\n  * `:timeout` - Stream timeout in ms (default: 15000)","ref":"FalEx.Streaming.html#stream/3-options"},{"type":"type","title":"FalEx.Streaming.t/0","doc":"","ref":"FalEx.Streaming.html#t:t/0"},{"type":"module","title":"FalEx.Types","doc":"Type definitions for the FalEx client library.","ref":"FalEx.Types.html"},{"type":"type","title":"FalEx.Types.completed_queue_status/0","doc":"","ref":"FalEx.Types.html#t:completed_queue_status/0"},{"type":"type","title":"FalEx.Types.endpoint_id/0","doc":"","ref":"FalEx.Types.html#t:endpoint_id/0"},{"type":"type","title":"FalEx.Types.input/0","doc":"","ref":"FalEx.Types.html#t:input/0"},{"type":"type","title":"FalEx.Types.log_entry/0","doc":"","ref":"FalEx.Types.html#t:log_entry/0"},{"type":"type","title":"FalEx.Types.metrics/0","doc":"","ref":"FalEx.Types.html#t:metrics/0"},{"type":"type","title":"FalEx.Types.output/0","doc":"","ref":"FalEx.Types.html#t:output/0"},{"type":"type","title":"FalEx.Types.queue_status/0","doc":"","ref":"FalEx.Types.html#t:queue_status/0"},{"type":"type","title":"FalEx.Types.queue_status_response/0","doc":"","ref":"FalEx.Types.html#t:queue_status_response/0"},{"type":"type","title":"FalEx.Types.request_id/0","doc":"","ref":"FalEx.Types.html#t:request_id/0"},{"type":"type","title":"FalEx.Types.result/0","doc":"","ref":"FalEx.Types.html#t:result/0"},{"type":"type","title":"FalEx.Types.run_options/0","doc":"","ref":"FalEx.Types.html#t:run_options/0"},{"type":"type","title":"FalEx.Types.storage_file/0","doc":"","ref":"FalEx.Types.html#t:storage_file/0"},{"type":"type","title":"FalEx.Types.url_options/0","doc":"","ref":"FalEx.Types.html#t:url_options/0"},{"type":"type","title":"FalEx.Types.validation_error_info/0","doc":"","ref":"FalEx.Types.html#t:validation_error_info/0"},{"type":"type","title":"FalEx.Types.webhook_response/0","doc":"","ref":"FalEx.Types.html#t:webhook_response/0"},{"type":"exception","title":"FalEx.Response.ApiError","doc":"Represents an API error response.","ref":"FalEx.Response.ApiError.html"},{"type":"exception","title":"FalEx.Response.ValidationError","doc":"Represents a validation error with field-specific details.","ref":"FalEx.Response.ValidationError.html"},{"type":"extras","title":"FalEx","doc":"# FalEx\n\nAn Elixir client for [fal.ai](https://fal.ai) - Serverless AI/ML inference platform.\n\n[![Hex.pm](https://img.shields.io/hexpm/v/fal_ex.svg)](https://hex.pm/packages/fal_ex)\n[![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/fal_ex)\n[![License](https://img.shields.io/hexpm/l/fal_ex.svg)](https://github.com/dantame/fal_ex/blob/main/LICENSE)","ref":"readme.html"},{"type":"extras","title":"About - FalEx","doc":"FalEx provides a robust and user-friendly Elixir client for seamless integration with fal.ai endpoints. It supports synchronous execution, queued jobs, streaming responses, real-time WebSocket connections, and automatic file handling.","ref":"readme.html#about"},{"type":"extras","title":"Features - FalEx","doc":"- 🚀 **Simple API** - Intuitive interface matching the JavaScript client\n- 📦 **Queue Support** - Submit long-running jobs and poll for results\n- 🌊 **Streaming** - Real-time streaming responses with Server-Sent Events\n- 🔌 **WebSocket** - Bidirectional real-time communication\n- 📤 **File Uploads** - Automatic file upload and URL transformation\n- 🔐 **Secure** - API key authentication with proxy support\n- 🧩 **Type Safe** - Full type specifications for better developer experience","ref":"readme.html#features"},{"type":"extras","title":"Installation - FalEx","doc":"Add `fal_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:fal_ex, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - FalEx","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"Application Configuration (Recommended) - FalEx","doc":"Configure FalEx in your application's config files:\n\n```elixir\n# config/config.exs\nconfig :fal_ex,\n  api_key: \"your-api-key\"\n\n# config/runtime.exs (for runtime configuration)\nimport Config\n\nif config_env() == :prod do\n  config :fal_ex,\n    api_key: System.fetch_env!(\"FAL_KEY\")\nend\n```","ref":"readme.html#application-configuration-recommended"},{"type":"extras","title":"Environment Variables - FalEx","doc":"You can also use environment variables (these override application config):\n\n```bash\n# Single API key\nexport FAL_KEY=\"your-api-key\"\n\n# Or using key ID and secret\nexport FAL_KEY_ID=\"your-key-id\"\nexport FAL_KEY_SECRET=\"your-key-secret\"\n```","ref":"readme.html#environment-variables"},{"type":"extras","title":"Programmatic Configuration - FalEx","doc":"Configure the client programmatically at runtime:\n\n```elixir\n# Configure globally\nFalEx.config(credentials: \"your-api-key\")\n\n# Or with more options\nFalEx.config(\n  credentials: {\"key-id\", \"key-secret\"},\n  proxy_url: \"/api/fal/proxy\"\n)\n```","ref":"readme.html#programmatic-configuration"},{"type":"extras","title":"Creating a Custom Client - FalEx","doc":"For more control, create a custom client instance:\n\n```elixir\nconfig = FalEx.Config.new(\n  credentials: System.get_env(\"FAL_KEY\"),\n  base_url: \"https://fal.run\"\n)\n\nclient = FalEx.Client.create(config)\n```","ref":"readme.html#creating-a-custom-client"},{"type":"extras","title":"Basic Usage - FalEx","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"Synchronous Execution - FalEx","doc":"Run models synchronously (blocks until complete):\n\n```elixir\n{:ok, result} = FalEx.run(\"fal-ai/fast-sdxl\",\n  input: %{\n    prompt: \"A serene mountain landscape at sunset\",\n    image_size: \"landscape_16_9\"\n  }\n)\n\nIO.inspect(result[\"images\"])\n```","ref":"readme.html#synchronous-execution"},{"type":"extras","title":"Queue-Based Execution - FalEx","doc":"For long-running models or when you want progress updates, use the queue system:\n\n```elixir\n{:ok, result} = FalEx.subscribe(\"fal-ai/flux/dev\",\n  input: %{\n    prompt: \"A detailed architectural drawing\",\n    num_images: 4\n  },\n  logs: true,\n  on_queue_update: fn status ->\n    IO.puts(\"Status: #{inspect(status.status)}\")\n    \n    # Print logs if available\n    Enum.each(status[:logs] || [], fn log ->\n      IO.puts(\"[#{log[\"level\"]}] #{log[\"message\"]}\")\n    end)\n  end\n)\n\n# Note: For fast models, the request may complete synchronously\n# and you'll get the result immediately with a single status update\n```","ref":"readme.html#queue-based-execution"},{"type":"extras","title":"Streaming Responses - FalEx","doc":"Stream partial results from compatible models (requires endpoints that support SSE streaming):\n\n```elixir\n# Get the stream\n{:ok, stream} = FalEx.stream(\"fal-ai/flux/dev\",\n  input: %{\n    prompt: \"A beautiful sunset over mountains\",\n    image_size: \"landscape_4_3\",\n    num_images: 1\n  }\n)\n\n# Process events as they arrive\nstream\n|> Stream.each(fn event ->\n  IO.inspect(event, label: \"Stream event\")\n  \n  # Events typically have a \"type\" field indicating progress\n  case event do\n    %{\"type\" => \"progress\", \"data\" => data} ->\n      IO.puts(\"Progress: #{inspect(data)}\")\n    \n    %{\"type\" => \"output\", \"data\" => data} ->\n      IO.puts(\"Got output: #{inspect(data[\"images\"])}\")\n      \n    _ ->\n      :ok\n  end\nend)\n|> Stream.run()\n```","ref":"readme.html#streaming-responses"},{"type":"extras","title":"Real-time WebSocket - FalEx","doc":"For bidirectional communication:\n\n```elixir\n{:ok, conn} = FalEx.realtime()\n|> FalEx.Realtime.connect(\"fal-ai/camera-turbo-stream\",\n  on_message: fn message ->\n    IO.inspect(message, label: \"Received\")\n  end,\n  on_error: fn error ->\n    IO.puts(\"Error: #{error}\")\n  end\n)\n\n# Send messages\nFalEx.Realtime.send(conn, %{\n  type: \"frame\",\n  data: base64_encoded_image\n})\n```","ref":"readme.html#real-time-websocket"},{"type":"extras","title":"File Handling - FalEx","doc":"FalEx automatically handles file uploads:\n\n```elixir\n# Automatic file upload from path\n{:ok, result} = FalEx.run(\"fal-ai/face-to-sticker\",\n  input: %{\n    image_url: \"/path/to/local/image.jpg\",  # Automatically uploaded\n    prompt: \"cartoon style\"\n  }\n)\n\n# Direct binary upload\nimage_data = File.read!(\"avatar.png\")\n{:ok, result} = FalEx.run(\"fal-ai/image-to-image\",\n  input: %{\n    image: image_data,  # Automatically uploaded\n    prompt: \"enhance details\"\n  }\n)\n\n# Manual upload\n{:ok, url} = FalEx.storage()\n|> FalEx.Storage.upload(image_data,\n  file_name: \"avatar.png\",\n  content_type: \"image/png\"\n)\n```","ref":"readme.html#file-handling"},{"type":"extras","title":"Advanced Usage - FalEx","doc":"","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Queue Operations - FalEx","doc":"Direct queue API access:\n\n```elixir\nqueue = FalEx.queue()\n\n# Submit a job\n{:ok, %{\"request_id\" => request_id}} = FalEx.Queue.submit(queue, \"fal-ai/fast-sdxl\",\n  input: %{prompt: \"A lighthouse\"},\n  webhook_url: \"https://example.com/webhook\"\n)\n\n# Check status\n{:ok, status} = FalEx.Queue.status(queue, \"fal-ai/fast-sdxl\",\n  request_id: request_id,\n  logs: true\n)\n\n# Cancel if needed\n:ok = FalEx.Queue.cancel(queue, \"fal-ai/fast-sdxl\",\n  request_id: request_id\n)\n\n# Get result when complete\n{:ok, result} = FalEx.Queue.result(queue, \"fal-ai/fast-sdxl\",\n  request_id: request_id\n)\n```","ref":"readme.html#queue-operations"},{"type":"extras","title":"Custom Middleware - FalEx","doc":"Add custom request/response handling:\n\n```elixir\ndefmodule MyApp.FalMiddleware do\n  @behaviour Tesla.Middleware\n  \n  def call(env, next, _opts) do\n    # Add custom header\n    env = Tesla.put_header(env, \"x-custom-header\", \"value\")\n    \n    # Call next middleware\n    case Tesla.run(env, next) do\n      {:ok, env} ->\n        # Log response\n        Logger.info(\"FAL Response: #{env.status}\")\n        {:ok, env}\n        \n      error ->\n        error\n    end\n  end\nend\n\n# Use in config\nFalEx.config(\n  credentials: \"...\",\n  request_middleware: MyApp.FalMiddleware\n)\n```","ref":"readme.html#custom-middleware"},{"type":"extras","title":"Error Handling - FalEx","doc":"FalEx provides detailed error types:\n\n```elixir\ncase FalEx.run(\"fal-ai/fast-sdxl\", input: %{}) do\n  {:ok, result} ->\n    IO.inspect(result.data)\n    \n  {:error, %FalEx.Response.ValidationError{} = error} ->\n    IO.puts(\"Validation failed: #{error.message}\")\n    \n    Enum.each(error.errors, fn error ->\n      IO.puts(\"  #{Enum.join(error.loc, \".\")}: #{error.msg}\")\n    end)\n    \n  {:error, %FalEx.Response.ApiError{} = error} ->\n    IO.puts(\"API error (#{error.status}): #{error.message}\")\n    \n  {:error, reason} ->\n    IO.puts(\"Unexpected error: #{inspect(reason)}\")\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Proxy Configuration - FalEx","doc":"For production deployments, use a proxy to protect your API keys:\n\n```elixir\n# Configure client to use proxy\nFalEx.config(proxy_url: \"/api/fal/proxy\")\n\n# In your Phoenix app, add a proxy endpoint\ndefmodule MyAppWeb.FalProxyController do\n  use MyAppWeb, :controller\n  \n  def proxy(conn, params) do\n    # Forward request to fal.ai with server-side credentials\n    # Implementation depends on your setup\n  end\nend\n```","ref":"readme.html#proxy-configuration"},{"type":"extras","title":"Examples - FalEx","doc":"","ref":"readme.html#examples"},{"type":"extras","title":"Text to Image - FalEx","doc":"```elixir\n{:ok, result} = FalEx.run(\"fal-ai/fast-sdxl\",\n  input: %{\n    prompt: \"A futuristic city with flying cars\",\n    negative_prompt: \"blurry, low quality\",\n    image_size: \"square_hd\",\n    num_inference_steps: 25,\n    guidance_scale: 7.5\n  }\n)\n\n[image | _] = result.data[\"images\"]\nIO.puts(\"Generated image: #{image[\"url\"]}\")\n```","ref":"readme.html#text-to-image"},{"type":"extras","title":"Image to Image - FalEx","doc":"```elixir\nsource_image = File.read!(\"source.jpg\")\n\n{:ok, result} = FalEx.run(\"fal-ai/image-to-image\",\n  input: %{\n    image: source_image,\n    prompt: \"Transform to watercolor painting\",\n    strength: 0.8\n  }\n)\n```","ref":"readme.html#image-to-image"},{"type":"extras","title":"LLM Streaming - FalEx","doc":"```elixir\nFalEx.stream(\"fal-ai/llama-3\",\n  input: %{\n    prompt: \"Explain quantum computing\",\n    max_tokens: 500,\n    temperature: 0.7\n  }\n)\n|> Stream.scan(\"\", fn chunk, acc ->\n  text = chunk[\"text\"] || \"\"\n  IO.write(text)\n  acc <> text\nend)\n|> Enum.to_list()\n|> List.last()\n```","ref":"readme.html#llm-streaming"},{"type":"extras","title":"Whisper Transcription - FalEx","doc":"```elixir\naudio_data = File.read!(\"recording.mp3\")\n\n{:ok, result} = FalEx.run(\"fal-ai/whisper\",\n  input: %{\n    audio: audio_data,\n    language: \"en\",\n    task: \"transcribe\",\n    include_timestamps: true\n  }\n)\n\nIO.puts(\"Transcription: #{result.data[\"text\"]}\")\n```","ref":"readme.html#whisper-transcription"},{"type":"extras","title":"Available Models - FalEx","doc":"See the [fal.ai model gallery](https://fal.ai/models) for available models and their parameters.\n\nPopular models include:\n- `fal-ai/fast-sdxl` - Fast SDXL image generation\n- `fal-ai/llama-3` - Meta's Llama 3 language model\n- `fal-ai/whisper` - OpenAI's Whisper speech recognition\n- `fal-ai/stable-video` - Stable Video Diffusion\n- `fal-ai/face-to-sticker` - Convert faces to stickers\n- `fal-ai/image-to-image` - Image transformation","ref":"readme.html#available-models"},{"type":"extras","title":"Testing - FalEx","doc":"```bash\n# Run tests\nmix test\n\n# Run tests with coverage\nmix test --cover\n\n# Run dialyzer\nmix dialyzer\n\n# Run credo\nmix credo\n```","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - FalEx","doc":"Contributions are welcome! Please feel free to submit a Pull Request. For major changes, please open an issue first to discuss what you would like to change.\n\n1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Commit your changes (`git commit -m 'Add some amazing feature'`)\n4. Push to the branch (`git push origin feature/amazing-feature`)\n5. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - FalEx","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Acknowledgments - FalEx","doc":"- Inspired by the official [fal-js](https://github.com/fal-ai/fal-js) JavaScript client\n- Built with [Tesla](https://github.com/elixir-tesla/tesla) HTTP client\n- WebSocket support via [WebSockex](https://github.com/Azolo/websockex)","ref":"readme.html#acknowledgments"},{"type":"extras","title":"Support - FalEx","doc":"- 📚 [Documentation](https://hexdocs.pm/fal_ex)\n- 🐛 [Issue Tracker](https://github.com/dantame/fal_ex/issues)","ref":"readme.html#support"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}